searchNodes=[{"ref":"Gearbox.html","title":"Gearbox","type":"behaviour","doc":"Gearbox is a functional state machine with an easy-to-use API, inspired by both Fsm and Machinery. Gearbox does not run in a process, so there&#39;s no potential for a GenServer bottleneck. This way there&#39;s also less overhead as you won&#39;t need to setup a supervision tree/manage your state machine processes. Note: Gearbox is heavily inspired by Machinery, and also took inspiration from Fsm. Gearbox is very similar to Machinery in term of the API usage, however it differs in the ways below: Gearbox does not use a GenServer as a backing process. Since GenServer can be a potential bottleneck in a system, for that reason I think it&#39;s best to leave process management to users of the library. No before/after callbacks. Callback allow you to add side effects, but side effects violate Single Responsibility Principle, and that can bring surprises to your codebase (e.g: &quot;How come everytime this transition happens, X happens?&quot;). Gearbox nudges you to keep domain-logic callbacks close to your contexts/domain events. Gearbox still ships with a guard_transition/3 callback, as that is intrinsic to state machines. Gearbox does not ship with a Phoenix Dashboard view. A really cool and great concept, but more often than not it is not needed and the added dependency can prove more trouble than worth. Rationale Gearbox operates on the philosophy that it acts purely as a functional state machine, wherein it does not care where your state is store (e.g: Ecto, GenServer), all Gearbox does is to help you ensure state transitions happen the way you expect it to. In most cases like for example Order, it is very likely that you don&#39;t need a process for that. Just get the record out of the database, run it through Gearbox machine, then persist it back to database. In some rare cases where you need to have a stateful state machine, for example a traffic light that has an internal timer to shift from red (30s) -&gt; green (30s) -&gt; yellow (5s) -&gt; red, you are better off to use an Agent/GenServer where you have better control over backpressuring/ business logics. As of now, Gearbox does not provide a way to create events/actions in a state machine. This is because Gearbox is not a domain/context wrapper, Events and actions that can trigger a state change should reside closer to your contexts, therefore I urge users to group these events as domain events (contexts), rather than state machine events. Gearbox previously shipped with before_transition/3 and after_transition/3 in 0.1.0, but after some discussions I have decided to take a deliberate decision to remove callbacks. This is because callbacks by nature, allow you to add side effects, but side effects violate Single Responsibility Principle, and callbacks can often bring unintended surprises to your codebase (e.g: &quot;How come everytime this transition happens, X happens?&quot;). Therefore, Gearbox nudges you to keep domain/business-logic callbacks close to your contexts/domain events. Gearbox still ships with a guard_transition/3 callback, as that is intrinsic to state machines. Options :field - used to retrieve the state of the given struct. Defaults to :state :states - list of finite states in the state machine :initial - initial state of the struct, if struct has nil state to begin with. Defaults to the first item of :states :transitions - a map of possible transitions from current_state to next_state. * wildcard is allowed to indicate any states. Example defmodule Gearbox.Order do defstruct items: [], total: 0, status: nil end defmodule Gearbox.OrderMachine do use Gearbox, field: :status, states: ~w(pending_payment cancelled paid pending_collection refunded fulfilled), initial: &quot;pending_payment&quot;, transitions: %{ &quot;pending_payment&quot; =&gt; ~w(cancelled paid), &quot;paid&quot; =&gt; ~w(pending_collection refunded), } end iex&gt; alias Gearbox.Order # Your struct iex&gt; alias Gearbox.OrderMachine # Your machine iex&gt; Gearbox.transition(%Order{}, OrderMachine, &quot;paid&quot;) {:ok, %Gearbox.Order{items: [], status: &quot;paid&quot;, total: 0}} Ecto Example {:ok, order} = Gearbox.transition(%Order{status: &quot;pending_payment&quot;}, OrderMachine, &quot;paid&quot;) Repo.insert!(order) # or even %Order{status: &quot;pending_payment&quot;} |&gt; Gearbox.transition!(OrderMachine, &quot;paid&quot;) |&gt; Repo.insert!()"},{"ref":"Gearbox.html#c:guard_transition/3","title":"Gearbox.guard_transition/3","type":"callback","doc":"Add guard conditions before transitioning. The function receives struct as the first argument, the current state as the second argument, and the desired state as the last argument. You can guard on both from and to states, e.g: Every time %Order{} transits out of pending, do X Every time %Order{} transits into paid, do Y If this function returns a {:halt, reason}, execution of the transition will halt. Any other things will allow the transition to go through. Note: This hook only gets triggered if the transition is valid."},{"ref":"Gearbox.html#transition/3","title":"Gearbox.transition/3","type":"function","doc":"Transition a struct to a given state. returns an {:ok, updated_struct} or {:error, message} tuple."},{"ref":"Gearbox.html#transition!/3","title":"Gearbox.transition!/3","type":"function","doc":"Transition a struct to a given state. If transition is invalid, an InvalidTransitionError exception is raised. Uses Gearbox.transition/3 under the hood."},{"ref":"Gearbox.html#t:state/0","title":"Gearbox.state/0","type":"type","doc":""},{"ref":"Gearbox.InvalidTransitionError.html","title":"Gearbox.InvalidTransitionError","type":"exception","doc":"This error is raised when you use Gearbox.transition!/3. For a non-error raising variant, see Gearbox.transition/3"}]